"""Utility to follow progress of a BI task."""

from time import sleep

from rich.progress import BarColumn, Progress, TaskProgressColumn, TextColumn
from slices_bi_client.api.tasks_api import TasksApi

MAX_PROGRESS_UPDATES_PER_REQUEST = 40  # stay within the limit of how long a GET-url can become

progress_columns = [
    TextColumn("[progress.description]{task.description}"),
    BarColumn(),
    TaskProgressColumn(),
]


def follow_progress(
    bi_tasks_api: TasksApi,
    task_ids: list[str],
    task_description_prefixes: list[str],
    *,
    transient=False,
):
    """Periodically query BI tasks API to check on task while showing progress bar."""
    assert len(task_ids) == len(task_description_prefixes)

    with Progress(*progress_columns, transient=transient) as progress:
        bi_task_ids_to_progress_task_ids = {}
        bi_task_ids_to_description_prefixes = {}
        finished_task_ids = set()

        for task_id, task_description_prefix in zip(task_ids, task_description_prefixes, strict=True):
            progress_task_id = progress.add_task(task_description_prefix, total=None)
            bi_task_ids_to_progress_task_ids[task_id] = progress_task_id
            bi_task_ids_to_description_prefixes[task_id] = task_description_prefix

        while not progress.finished:
            assert len(finished_task_ids) < len(task_ids)

            tasks_to_query = [task_id for task_id in task_ids if task_id not in finished_task_ids]
            bi_task_progresses = []
            while tasks_to_query:
                bi_task_progresses.extend(
                    bi_tasks_api.get_tasks(
                        task_ids=tasks_to_query[0:MAX_PROGRESS_UPDATES_PER_REQUEST],  # type: ignore
                    )
                )
                tasks_to_query = tasks_to_query[MAX_PROGRESS_UPDATES_PER_REQUEST:]

            bi_task_progresses = bi_tasks_api.get_tasks(
                task_ids=[task_id for task_id in task_ids if task_id not in finished_task_ids],
            )

            for task_progress in bi_task_progresses:
                progress_task_id = bi_task_ids_to_progress_task_ids[task_progress.id]
                progress.update(
                    progress_task_id,
                    total=task_progress.total,
                    completed=task_progress.completed,
                )
                task_finished = task_progress.status in ("finished", "failed")
                if task_progress.description:
                    progress.update(
                        progress_task_id,
                        description=f"{bi_task_ids_to_description_prefixes[task_progress.id]}: "
                        f"{task_progress.description}",
                        visible=not task_finished,
                    )

                if task_finished:
                    finished_task_ids.add(progress_task_id)

            if not progress.finished:
                sleep(0.5)


def follow_one_progress(
    bi_tasks_api: TasksApi,
    task_id,
    task_description_prefix,
    *,
    transient=False,
):
    """Periodically query BI tasks API to check on task while showing progress bar."""
    follow_progress(bi_tasks_api, [task_id], [task_description_prefix], transient=transient)
