"""Slices CLI authentication."""

from datetime import UTC, datetime
from typing import Annotated, Optional

import humanize
import typer
from rich import print
from rich.table import Table

from .config import get_config_string, set_config_string
from .context import Context

project_app = typer.Typer(help="Projects", no_args_is_help=True)


@project_app.command(help="Show projects of which you are a member")
def list(
    ctx: Context,
    show_all: Annotated[bool, typer.Option(help="Also show expired projects")] = False,
    show_details: Annotated[bool, typer.Option(help="Show project IDs")] = False,
):
    """List projects."""
    projects = ctx.obj.api.projects_api.get_projects(include_expired=show_all)

    columns = [
        "Name",
        "Role",
        "Created At",
        "Expires At",
    ]

    if show_details:
        columns.insert(0, "ID")

    table = Table(*columns, title=f"Projects for {ctx.obj.get_logged_in_userinfo('preferred_username')}")
    for project in projects.projects:
        row = [
            project.name,
            project.role,
            project.created_at.astimezone().strftime("%Y-%m-%d %H:%M"),
            project.expires_at.astimezone().strftime("%Y-%m-%d %H:%M") if project.expires_at else "-",
        ]
        if show_details:
            row.insert(0, project.id)

        table.add_row(*row)

    print(table)


@project_app.command(help="Show the current project")
def show(
    ctx: Context,
):
    """Show the current project."""
    current_project_id = get_config_string("default", section="project")

    if not current_project_id:
        print("[red]No project set. :x:")
        raise typer.Exit

    project = ctx.obj.api.projects_api.get_project(current_project_id)

    print(f"[bold]The current project is '{project.name}'.")
    print(f":ticket: You are a {project.role} in this project.")

    if project.expires_at:
        humanized_delta = humanize.precisedelta(
            project.expires_at - datetime.now(UTC),
            minimum_unit="days",
            format="%0.0f",
        )
        if project.expires_at < datetime.now(UTC):
            print(
                f"[red]:clock10: It expired on {project.expires_at.astimezone().strftime('%Y-%m-%d %H:%M')}"
                f" ({humanized_delta} ago)."
            )
        else:
            print(
                f":clock10: It expires on {project.expires_at.astimezone().strftime('%Y-%m-%d %H:%M')}"
                f" (in {humanized_delta})"
            )
    else:
        print("[green]:clock10: It never expires.")


@project_app.command(help="Set the current project")
def use(
    ctx: Context,
    name: Annotated[str, typer.Argument(help="The ID or the name of the project that you want to set as default.")],
):
    """Set the current project."""

    projects = ctx.obj.api.projects_api.get_projects()

    project = None
    for candidate_project in projects.projects:
        if name in (candidate_project.id, candidate_project.name):
            project = candidate_project
            break

    if not project:
        print(f"[red]Could not find project '{name}'. :x:")
        print(f"The following projects are available: {', '.join(project.name for project in projects.projects)}")
        raise typer.Exit(1)

    set_config_string("default", project.id, "project")

    print(
        f"[green]The current project was set to '[bold]{project.name}[/]', "
        f"""in which you are a [bold]{project.role}[/] and which {
        f'expires on {project.expires_at.astimezone().strftime("%Y-%m-%d %H:%M")}'
        if project.expires_at
        else 'never expires'}."""
    )


@project_app.command(help="Clear the current project", hidden=True)
def clear():
    """Clear the current project."""

    set_config_string("default", None, "project")
    print("Cleared current project.")


@project_app.command(hidden=True, help="Get a JWT project token")
def token(
    ctx: Context,
    name: Annotated[
        Optional[str], typer.Argument(help="The ID or the name of the project that you want to set as default.")
    ] = None,
):
    """Get a JWT project token."""

    if name:
        projects = ctx.obj.api.projects_api.get_projects()

        project = None
        for candidate_project in projects.projects:
            if name in (candidate_project.id, candidate_project.name):
                project = candidate_project
                break

        if not project:
            print(f"[red]Could not find project '{name}'. :x:")
            print(f"The following projects are available: {', '.join(project.name for project in projects.projects)}")
            raise typer.Exit(1)

        current_project_id = project.id
    else:
        current_project_id = get_config_string("default", section="project")

        if not current_project_id:
            print("[red]No project set. :x:")
            raise typer.Exit(1)

    project_token = ctx.obj.api.get_project_jwt(current_project_id)

    print(project_token)
